@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_digest.h,v 1.1 2003/07/23 16:11:29 gotoyuzo Exp $
+ * $Id: ossl_digest.h 12496 2007-06-08 15:02:04Z technorama $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -11,11 +11,11 @@
 #if !defined(_OSSL_DIGEST_H_)
 #define _OSSL_DIGEST_H_
 
-extern VALUE mDigest;
 extern VALUE cDigest;
 extern VALUE eDigestError;
 
 const EVP_MD *GetDigestPtr(VALUE);
+VALUE ossl_digest_new(const EVP_MD *);
 void Init_ossl_digest(void);
 
 #endif /* _OSSL_DIGEST_H_ */
