@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509ext.c,v 1.8 2003/11/15 13:19:17 gotoyuzo Exp $
+ * $Id: ossl_x509ext.c 28004 2010-05-24 23:58:49Z shyouhei $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -26,9 +26,12 @@
     OSSL_Check_Kind(obj, cX509Ext); \
     GetX509Ext(obj, ext); \
 } while (0)
-
-#define MakeX509ExtFactory(klass, obj, ctx) \
-    obj = Data_Make_Struct(klass, X509V3_CTX, 0, ossl_x509extfactory_free, ctx)
+#define MakeX509ExtFactory(klass, obj, ctx) do { \
+    if (!(ctx = OPENSSL_malloc(sizeof(X509V3_CTX)))) \
+        ossl_raise(rb_eRuntimeError, "CTX wasn't allocated!"); \
+    X509V3_set_ctx(ctx, NULL, NULL, NULL, NULL, 0); \
+    obj = Data_Wrap_Struct(klass, 0, ossl_x509extfactory_free, ctx); \
+} while (0)
 #define GetX509ExtFactory(obj, ctx) do { \
     Data_Get_Struct(obj, X509V3_CTX, ctx); \
     if (!ctx) { \
@@ -195,6 +198,7 @@ ossl_x509extfactory_initialize(int argc, VALUE *argv, VALUE self)
 	ossl_x509extfactory_set_subject_req(self, subject_req);
     if (!NIL_P(crl))
 	ossl_x509extfactory_set_crl(self, crl);
+    rb_iv_set(self, "@config", Qnil);
 
     return self;
 }
@@ -214,22 +218,35 @@ ossl_x509extfactory_create_ext(int argc, VALUE *argv, VALUE self)
     X509_EXTENSION *ext;
     VALUE oid, value, critical, valstr, obj;
     int nid;
+#ifdef HAVE_X509V3_EXT_NCONF_NID
+    VALUE rconf;
+    CONF *conf;
+#else
+    static LHASH *empty_lhash;
+#endif
 
-    GetX509ExtFactory(self, ctx);
     rb_scan_args(argc, argv, "21", &oid, &value, &critical);
     StringValue(oid);
     StringValue(value);
     if(NIL_P(critical)) critical = Qfalse;
 
-    nid = OBJ_ln2nid(RSTRING(oid)->ptr);
-    if(!nid) nid = OBJ_sn2nid(RSTRING(oid)->ptr);
-    if(!nid) ossl_raise(eX509ExtError, "unknown OID `%s'", RSTRING(oid)->ptr);
+    nid = OBJ_ln2nid(RSTRING_PTR(oid));
+    if(!nid) nid = OBJ_sn2nid(RSTRING_PTR(oid));
+    if(!nid) ossl_raise(eX509ExtError, "unknown OID `%s'", RSTRING_PTR(oid));
     valstr = rb_str_new2(RTEST(critical) ? "critical," : "");
     rb_str_append(valstr, value);
-    ext = X509V3_EXT_conf_nid(NULL, ctx, nid, RSTRING(valstr)->ptr);
+    GetX509ExtFactory(self, ctx);
+#ifdef HAVE_X509V3_EXT_NCONF_NID
+    rconf = rb_iv_get(self, "@config");
+    conf = NIL_P(rconf) ? NULL : GetConfigPtr(rconf);
+    ext = X509V3_EXT_nconf_nid(conf, ctx, nid, RSTRING_PTR(valstr));
+#else
+    if (!empty_lhash) empty_lhash = lh_new(NULL, NULL);
+    ext = X509V3_EXT_conf_nid(empty_lhash, ctx, nid, RSTRING_PTR(valstr));
+#endif
     if (!ext){
 	ossl_raise(eX509ExtError, "%s = %s",
-		   RSTRING(oid)->ptr, RSTRING(value)->ptr);
+		   RSTRING_PTR(oid), RSTRING_PTR(value));
     }
     WrapX509Ext(cX509Ext, obj, ext);
 
@@ -260,13 +277,13 @@ ossl_x509ext_initialize(int argc, VALUE *argv, VALUE self)
     unsigned char *p;
     X509_EXTENSION *ext;
 
+    GetX509Ext(self, ext);
     if(rb_scan_args(argc, argv, "12", &oid, &value, &critical) == 1){
-	/* evaluate oid as a DER string */
 	oid = ossl_to_der_if_possible(oid);
 	StringValue(oid);
-	GetX509Ext(self, ext);
-	p  = RSTRING(oid)->ptr;
-	if(!d2i_X509_EXTENSION(&ext, &p, RSTRING(oid)->len))
+	p  = RSTRING_PTR(oid);
+	if(!d2i_X509_EXTENSION((X509_EXTENSION**)&DATA_PTR(self),
+			       &p, RSTRING_LEN(oid)))
 	    ossl_raise(eX509ExtError, NULL);
 	return self;
     }
@@ -284,11 +301,11 @@ ossl_x509ext_set_oid(VALUE self, VALUE oid)
     ASN1_OBJECT *obj;
     char *s;
 
-    GetX509Ext(self, ext);
     s = StringValuePtr(oid);
     obj = OBJ_txt2obj(s, 0);
     if(!obj) obj = OBJ_txt2obj(s, 1);
     if(!obj) ossl_raise(eX509ExtError, NULL);
+    GetX509Ext(self, ext);
     X509_EXTENSION_set_object(ext, obj);
 
     return oid;
@@ -301,21 +318,22 @@ ossl_x509ext_set_value(VALUE self, VALUE data)
     ASN1_OCTET_STRING *asn1s;
     char *s;
 
-    GetX509Ext(self, ext);
     data = ossl_to_der_if_possible(data);
     StringValue(data);
-    if(!(s = OPENSSL_malloc(RSTRING(data)->len)))
+    if(!(s = OPENSSL_malloc(RSTRING_LEN(data))))
 	ossl_raise(eX509ExtError, "malloc error");
-    memcpy(s, RSTRING(data)->ptr, RSTRING(data)->len);
+    memcpy(s, RSTRING_PTR(data), RSTRING_LEN(data));
     if(!(asn1s = ASN1_OCTET_STRING_new())){
-	free(s);
+        OPENSSL_free(s);
 	ossl_raise(eX509ExtError, NULL);
     }
-    if(!M_ASN1_OCTET_STRING_set(asn1s, s, RSTRING(data)->len)){
-	free(s);
+    if(!M_ASN1_OCTET_STRING_set(asn1s, s, RSTRING_LEN(data))){
+        OPENSSL_free(s);
 	ASN1_OCTET_STRING_free(asn1s);
 	ossl_raise(eX509ExtError, NULL);
     }
+    OPENSSL_free(s);
+    GetX509Ext(self, ext);
     X509_EXTENSION_set_data(ext, asn1s);
 
     return data;
@@ -376,6 +394,7 @@ static VALUE
 ossl_x509ext_get_critical(VALUE obj)
 {
     X509_EXTENSION *ext;
+
     GetX509Ext(obj, ext);
     return X509_EXTENSION_get_critical(ext) ? Qtrue : Qfalse;
 }
@@ -392,7 +411,7 @@ ossl_x509ext_to_der(VALUE obj)
     if((len = i2d_X509_EXTENSION(ext, NULL)) <= 0)
 	ossl_raise(eX509ExtError, NULL);
     str = rb_str_new(0, len);
-    p = RSTRING(str)->ptr;
+    p = RSTRING_PTR(str);
     if(i2d_X509_EXTENSION(ext, &p) < 0)
 	ossl_raise(eX509ExtError, NULL);
     ossl_str_adjust(str, p);
