@@ -61,10 +61,15 @@
 #undef NI_NUMERICSERV
 #undef NI_DGRAM
 
+#undef addrinfo
 #define addrinfo addrinfo__compat
+#undef getaddrinfo
 #define getaddrinfo getaddrinfo__compat
+#undef getnameinfo
 #define getnameinfo getnameinfo__compat
+#undef freehostent
 #define freehostent freehostent__compat
+#undef freeaddrinfo
 #define freeaddrinfo freeaddrinfo__compat
 
 #ifndef __P
@@ -115,8 +120,10 @@
 /*
  * Constants for getnameinfo()
  */
+#ifndef NI_MAXHOST
 #define	NI_MAXHOST	1025
 #define	NI_MAXSERV	32
+#endif
 
 /*
  * Flag values for getnameinfo()
@@ -153,7 +160,7 @@ extern int getnameinfo __P((
 	int flags));
 
 extern void freehostent __P((struct hostent *));
-extern void freeaddrent __P((struct addrinfo *));
+extern void freeaddrinfo __P((struct addrinfo *));
 #if defined __UCLIBC__
 const
 #endif
