@@ -39,8 +39,11 @@
  */
 
 #include "config.h"
+#ifdef RUBY_EXTCONF_H
+#include RUBY_EXTCONF_H
+#endif
 #include <sys/types.h>
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(__VMS)
 #include <sys/param.h>
 #if defined(__BEOS__)
 # include <net/socket.h>
@@ -56,11 +59,23 @@
 #endif
 #include <netdb.h>
 #if defined(HAVE_RESOLV_H)
+#ifdef _SX
+#include <stdio.h>
+#endif
 #include <resolv.h>
 #endif
 #include <unistd.h>
+#elif defined(__VMS )
+#include <socket.h>
+#include <inet.h>
+#include <in.h>
+#include <netdb.h>
 #else
+#if USE_WINSOCK2
 #include <winsock2.h>
+#else
+#include <winsock.h>
+#endif
 #include <io.h>
 #endif
 #include <string.h>
@@ -191,7 +206,8 @@ if (pai->ai_flags & AI_CANONNAME) {\
 
 #define ERR(err) { error = (err); goto bad; }
 
-#if defined __UCLIBC__
+#ifndef HAVE_GAI_STRERROR
+#ifdef GAI_STRERROR_CONST
 const
 #endif
 char *
@@ -202,6 +218,7 @@ gai_strerror(ecode)
 		ecode = EAI_MAX;
 	return (char *)ai_errlist[ecode];
 }
+#endif
 
 void
 freeaddrinfo(ai)
@@ -380,7 +397,7 @@ getaddrinfo(hostname, servname, hints, res)
 			port = htons((unsigned short)atoi(servname));
 		} else {
 			struct servent *sp;
-			char *proto;
+			const char *proto;
 
 			proto = NULL;
 			switch (pai->ai_socktype) {
