@@ -1,3 +1,5 @@
+# -*- coding: euc-jp -*-
+#
 # labelframe.rb
 #
 # This demonstration script creates a toplevel window containing
@@ -17,10 +19,12 @@ $labelframe_demo = TkToplevel.new {|w|
   positionWindow(w)
 }
 
+base_frame = TkFrame.new($labelframe_demo).pack(:fill=>:both, :expand=>true)
+
 # Some information
-TkLabel.new($labelframe_demo, 
-	    :font=>$font, :wraplength=>'4i', :justify=>:left, 
-	    :text=><<EOL).pack(:side=>:top)
+TkLabel.new(base_frame, 
+            :font=>$font, :wraplength=>'4i', :justify=>:left, 
+            :text=><<EOL).pack(:side=>:top)
 TkLabelFrame åȤϴϢ widget 
 ޤȤƼ갷Ѥޤ
 ٥̾ʸǤⲿ餫Υå
@@ -34,32 +38,32 @@ labelframe åȤƤʤ
 EOL
 
 # The bottom buttons
-TkFrame.new($labelframe_demo){|f|
+TkFrame.new(base_frame){|f|
   pack(:side=>:bottom, :fill=>:x, :pady=>'2m')
 
-  TkButton.new(f, :text=>'λ', :width=>15, :command=>proc{
-		 $labelframe_demo.destroy
-		 $labelframe_demo = nil
-	       }).pack(:side=>:left, :expand=>true)
+  TkButton.new(f, :text=>'Ĥ', :width=>15, :command=>proc{
+                 $labelframe_demo.destroy
+                 $labelframe_demo = nil
+               }).pack(:side=>:left, :expand=>true)
 
   TkButton.new(f, :text=>'ɻ', :width=>15, :command=>proc{
-		 showCode 'labelframe'
-	       }).pack(:side=>:left, :expand=>true)
+                 showCode 'labelframe'
+               }).pack(:side=>:left, :expand=>true)
 }
 
 # Demo area
-w = TkFrame.new($labelframe_demo).pack(:side=>:bottom, :fill=>:both, 
-				       :expand=>true)
+w = TkFrame.new(base_frame).pack(:side=>:bottom, :fill=>:both, 
+                                       :expand=>true)
 
 # A group of radiobuttons in a labelframe
 TkLabelFrame.new(w, :text=>'', 
-		 :padx=>2, :pady=>2) {|f|
+                 :padx=>2, :pady=>2) {|f|
   grid(:row=>0, :column=>0, :pady=>'2m', :padx=>'2m')
 
   v = TkVariable.new
   (1..4).each{|i|
     TkRadiobutton.new(f, :text=>"This is value #{i}", 
-		      :variable=>v, :value=>i) {
+                      :variable=>v, :value=>i) {
       pack(:side=>:top, :fill=>:x, :pady=>2)
     }
   }
@@ -82,7 +86,7 @@ end
 
 TkLabelFrame.new(w, :pady=>2, :padx=>2){|f|
   TkCheckButton.new(f, :widgetname=>'cb', :variable=>$lfdummy, 
-		    :text=>"ץ", :padx=>0) {|cb|
+                    :text=>"ץ", :padx=>0) {|cb|
     command proc{lfEnableButtons(f)}
     f.labelwidget(cb)
   }
