@@ -3,7 +3,7 @@
  *
  *   Copyright (C) UENO Katsuhiro 2000-2003
  *
- * $Id: zlib.c,v 1.7.2.12 2004/12/18 07:37:01 nobu Exp $
+ * $Id: zlib.c 31714 2011-05-23 04:49:42Z shyouhei $
  */
 
 #include <ruby.h>
@@ -39,6 +39,7 @@ static VALUE rb_zlib_crc32 _((int, VALUE*, VALUE));
 static VALUE rb_zlib_crc_table _((VALUE));
 static voidpf zlib_mem_alloc _((voidpf, uInt, uInt));
 static void zlib_mem_free _((voidpf, voidpf));
+static void finalizer_warn _((const char*));
 
 struct zstream;
 struct zstream_funcs;
@@ -55,13 +56,14 @@ static void zstream_reset_input _((struct zstream*));
 static void zstream_passthrough_input _((struct zstream*));
 static VALUE zstream_detach_input _((struct zstream*));
 static void zstream_reset _((struct zstream*));
-static void zstream_end _((struct zstream*));
+static VALUE zstream_end _((struct zstream*));
 static void zstream_run _((struct zstream*, Bytef*, uInt, int));
 static VALUE zstream_sync _((struct zstream*, Bytef*, uInt));
 static void zstream_mark _((struct zstream*));
 static void zstream_free _((struct zstream*));
 static VALUE zstream_new _((VALUE, const struct zstream_funcs*));
 static struct zstream *get_zstream _((VALUE));
+static void zstream_finalize _((struct zstream*));
 
 static VALUE rb_zstream_end _((VALUE));
 static VALUE rb_zstream_reset _((VALUE));
@@ -80,7 +82,8 @@ static VALUE rb_zstream_closed_p _((VALUE));
 
 static VALUE rb_deflate_s_allocate _((VALUE));
 static VALUE rb_deflate_initialize _((int, VALUE*, VALUE));
-static VALUE rb_deflate_clone _((VALUE));
+static VALUE rb_deflate_init_copy _((VALUE, VALUE));
+static VALUE deflate_run _((VALUE));
 static VALUE rb_deflate_s_deflate _((int, VALUE*, VALUE));
 static void do_deflate _((struct zstream*, VALUE, int));
 static VALUE rb_deflate_deflate _((int, VALUE*, VALUE));
@@ -89,6 +92,7 @@ static VALUE rb_deflate_flush _((int, VALUE*, VALUE));
 static VALUE rb_deflate_params _((VALUE, VALUE, VALUE));
 static VALUE rb_deflate_set_dictionary _((VALUE, VALUE));
 
+static VALUE inflate_run _((VALUE));
 static VALUE rb_inflate_s_allocate _((VALUE));
 static VALUE rb_inflate_initialize _((int, VALUE*, VALUE));
 static VALUE rb_inflate_s_inflate _((VALUE, VALUE));
@@ -123,8 +127,9 @@ static void gzfile_calc_crc _((struct gzfile*, VALUE));
 static VALUE gzfile_read _((struct gzfile*, int));
 static VALUE gzfile_read_all _((struct gzfile*));
 static void gzfile_ungetc _((struct gzfile*, int));
-static VALUE gzfile_finalize _((VALUE));
+static VALUE gzfile_writer_end_run _((VALUE));
 static void gzfile_writer_end _((struct gzfile*));
+static VALUE gzfile_reader_end_run _((VALUE));
 static void gzfile_reader_end _((struct gzfile*));
 static void gzfile_reader_rewind _((struct gzfile*));
 static VALUE gzfile_reader_get_unused _((struct gzfile*));
@@ -237,6 +242,15 @@ raise_zlib_error(err, msg)
 }
 
 
+/*--- Warning (in finalizer) ---*/
+
+static void
+finalizer_warn(msg)
+    const char *msg;
+{
+    fprintf(stderr, "zlib(finalizer): %s\n", msg);
+}
+
 
 /*-------- module Zlib --------*/
 
@@ -288,7 +302,7 @@ do_checksum(argc, argv, func)
 /*
  * call-seq: Zlib.adler32(string, adler)
  *
- * Calculates Alder-32 checksum for +string+, and returns updated value of
+ * Calculates Adler-32 checksum for +string+, and returns updated value of
  * +adler+. If +string+ is omitted, it returns the Adler-32 initial value. If
  * +adler+ is omitted, it assumes that the initial value is given to +adler+.
  *
@@ -361,15 +375,13 @@ struct zstream {
 #define ZSTREAM_FLAG_READY      0x1
 #define ZSTREAM_FLAG_IN_STREAM  0x2
 #define ZSTREAM_FLAG_FINISHED   0x4
-#define ZSTREAM_FLAG_FINALIZE   0x8
-#define ZSTREAM_FLAG_CLOSED     0x10
-#define ZSTREAM_FLAG_UNUSED     0x20
+#define ZSTREAM_FLAG_CLOSING    0x8
+#define ZSTREAM_FLAG_UNUSED     0x10
 
 #define ZSTREAM_READY(z)       ((z)->flags |= ZSTREAM_FLAG_READY)
 #define ZSTREAM_IS_READY(z)    ((z)->flags & ZSTREAM_FLAG_READY)
 #define ZSTREAM_IS_FINISHED(z) ((z)->flags & ZSTREAM_FLAG_FINISHED)
-#define ZSTREAM_IS_FINALIZE(z) ((z)->flags & ZSTREAM_FLAG_FINALIZE)
-#define ZSTREAM_IS_CLOSED(z)   ((z)->flags & ZSTREAM_FLAG_CLOSED)
+#define ZSTREAM_IS_CLOSING(z)  ((z)->flags & ZSTREAM_FLAG_CLOSING)
 
 /* I think that more better value should be found,
    but I gave up finding it. B) */
@@ -598,7 +610,8 @@ zstream_append_input(z, src, len)
 }
 
 #define zstream_append_input2(z,v)\
-    zstream_append_input((z), RSTRING(v)->ptr, RSTRING(v)->len)
+    RB_GC_GUARD(v),\
+    zstream_append_input((z), (Bytef*)RSTRING_PTR(v), RSTRING_LEN(v))
 
 static void
 zstream_discard_input(z, len)
@@ -656,7 +669,7 @@ zstream_reset(z)
     int err;
 
     err = z->func->reset(&z->stream);
-    if (err != Z_OK && !ZSTREAM_IS_FINALIZE(z)) {
+    if (err != Z_OK) {
 	raise_zlib_error(err, z->stream.msg);
     }
     z->flags = ZSTREAM_FLAG_READY;
@@ -667,31 +680,28 @@ zstream_reset(z)
     zstream_reset_input(z);
 }
 
-static void
+static VALUE
 zstream_end(z)
     struct zstream *z;
 {
     int err;
 
-    if (!ZSTREAM_IS_READY(z) && !ZSTREAM_IS_FINALIZE(z)) {
-	if (RTEST(ruby_debug)) {
-	    rb_warning("attempt to close uninitialized zstream; ignored.");
-	}
-	return;
+    if (!ZSTREAM_IS_READY(z)) {
+	rb_warning("attempt to close uninitialized zstream; ignored.");
+	return Qnil;
     }
     if (z->flags & ZSTREAM_FLAG_IN_STREAM) {
-	if (RTEST(ruby_debug)) {
-	    rb_warning("attempt to close unfinished zstream; reset forced.");
-	}
+	rb_warning("attempt to close unfinished zstream; reset forced.");
 	zstream_reset(z);
     }
 
     zstream_reset_input(z);
     err = z->func->end(&z->stream);
-    if (err != Z_OK && !ZSTREAM_IS_FINALIZE(z)) {
+    if (err != Z_OK) {
 	raise_zlib_error(err, z->stream.msg);
     }
     z->flags = 0;
+    return Qnil;
 }
 
 static void
@@ -703,6 +713,7 @@ zstream_run(z, src, len, flush)
 {
     uInt n;
     int err;
+    volatile VALUE guard;
 
     if (NIL_P(z->input) && len == 0) {
 	z->stream.next_in = "";
@@ -712,6 +723,10 @@ zstream_run(z, src, len, flush)
 	zstream_append_input(z, src, len);
 	z->stream.next_in = RSTRING(z->input)->ptr;
 	z->stream.avail_in = RSTRING(z->input)->len;
+	/* keep reference to `z->input' so as not to be garbage collected
+	   after zstream_reset_input() and prevent `z->stream.next_in'
+	   from dangling. */
+	guard = z->input;
     }
 
     if (z->stream.avail_out == 0) {
@@ -719,6 +734,9 @@ zstream_run(z, src, len, flush)
     }
 
     for (;;) {
+        /* VC allocates err and guard to same address.  accessing err and guard
+           in same scope prevents it. */
+        RB_GC_GUARD(guard);
 	n = z->stream.avail_out;
 	err = z->func->run(&z->stream, flush);
 	z->buf_filled += n - z->stream.avail_out;
@@ -751,6 +769,7 @@ zstream_run(z, src, len, flush)
     zstream_reset_input(z);
     if (z->stream.avail_in > 0) {
 	zstream_append_input(z, z->stream.next_in, z->stream.avail_in);
+        guard = Qnil; /* prevent tail call to make guard effective */
     }
 }
 
@@ -804,12 +823,24 @@ zstream_mark(z)
     rb_gc_mark(z->input);
 }
 
+static void
+zstream_finalize(z)
+    struct zstream *z;
+{
+    int err = z->func->end(&z->stream);
+    if (err == Z_STREAM_ERROR)
+	finalizer_warn("the stream state was inconsistent.");
+    if (err == Z_DATA_ERROR)
+	finalizer_warn("the stream was freed prematurely.");
+}
+
 static void
 zstream_free(z)
     struct zstream *z;
 {
-    z->flags |= ZSTREAM_FLAG_FINALIZE;
-    zstream_end(z);
+    if (ZSTREAM_IS_READY(z)) {
+	zstream_finalize(z);
+    }
     free(z);
 }
 
@@ -1152,26 +1183,36 @@ rb_deflate_initialize(argc, argv, obj)
  * Duplicates the deflate stream.
  */
 static VALUE
-rb_deflate_clone(obj)
-    VALUE obj;
+rb_deflate_init_copy(self, orig)
+    VALUE self, orig;
 {
-    struct zstream *z = get_zstream(obj);
-    struct zstream *z2;
-    VALUE clone;
+    struct zstream *z1, *z2;
     int err;
 
-    clone = zstream_deflate_new(rb_class_of(obj));
-    Data_Get_Struct(clone, struct zstream, z2);
+    Data_Get_Struct(self, struct zstream, z1);
+    z2 = get_zstream(orig);
 
-    err = deflateCopy(&z2->stream, &z->stream);
+    err = deflateCopy(&z1->stream, &z2->stream);
     if (err != Z_OK) {
 	raise_zlib_error(err, 0);
     }
+    z1->input = NIL_P(z2->input) ? Qnil : rb_str_dup(z2->input);
+    z1->buf   = NIL_P(z2->buf)   ? Qnil : rb_str_dup(z2->buf);
+    z1->buf_filled = z2->buf_filled;
+    z1->flags = z2->flags;
 
-    z2->flags = z->flags;
-    CLONESETUP(clone, obj);
-    OBJ_INFECT(clone, obj);
-    return clone;
+    return self;
+}
+
+static VALUE
+deflate_run(args)
+    VALUE args;
+{
+    struct zstream *z = (struct zstream *)((VALUE *)args)[0];
+    VALUE src = ((VALUE *)args)[1];
+
+    zstream_run(z, RSTRING(src)->ptr, RSTRING(src)->len, Z_FINISH);
+    return zstream_detach_buffer(z);
 }
 
 /*
@@ -1201,7 +1242,7 @@ rb_deflate_s_deflate(argc, argv, klass)
     VALUE klass;
 {
     struct zstream z;
-    VALUE src, level, dst;
+    VALUE src, level, dst, args[2];
     int err, lev;
 
     rb_scan_args(argc, argv, "11", &src, &level);
@@ -1215,9 +1256,9 @@ rb_deflate_s_deflate(argc, argv, klass)
     }
     ZSTREAM_READY(&z);
 
-    zstream_run(&z, RSTRING(src)->ptr, RSTRING(src)->len, Z_FINISH);
-    dst = zstream_detach_buffer(&z);
-    zstream_end(&z);
+    args[0] = (VALUE)&z;
+    args[1] = src;
+    dst = rb_ensure(deflate_run, (VALUE)args, zstream_end, (VALUE)&z);
 
     OBJ_INFECT(dst, src);
     return dst;
@@ -1337,11 +1378,10 @@ rb_deflate_params(obj, v_level, v_strategy)
     level = ARG_LEVEL(v_level);
     strategy = ARG_STRATEGY(v_strategy);
 
+    zstream_run(z, (Bytef*)"", 0, Z_SYNC_FLUSH);
     err = deflateParams(&z->stream, level, strategy);
     while (err == Z_BUF_ERROR) {
-	if (RTEST(ruby_debug)) {
-	    rb_warning("deflateParams() returned Z_BUF_ERROR");
-	}
+	rb_warning("deflateParams() returned Z_BUF_ERROR");
 	zstream_expand_buffer(z);
 	err = deflateParams(&z->stream, level, strategy);
     }
@@ -1430,6 +1470,18 @@ rb_inflate_initialize(argc, argv, obj)
     return obj;
 }
 
+static VALUE
+inflate_run(args)
+    VALUE args;
+{
+    struct zstream *z = (struct zstream *)((VALUE *)args)[0];
+    VALUE src = ((VALUE *)args)[1];
+
+    zstream_run(z, RSTRING(src)->ptr, RSTRING(src)->len, Z_SYNC_FLUSH);
+    zstream_run(z, "", 0, Z_FINISH);  /* for checking errors */
+    return zstream_detach_buffer(z);
+}
+
 /*
  * call-seq: Zlib::Inflate.inflate(string)
  *
@@ -1452,7 +1504,7 @@ rb_inflate_s_inflate(obj, src)
     VALUE obj, src;
 {
     struct zstream z;
-    VALUE dst;
+    VALUE dst, args[2];
     int err;
 
     StringValue(src);
@@ -1463,10 +1515,9 @@ rb_inflate_s_inflate(obj, src)
     }
     ZSTREAM_READY(&z);
 
-    zstream_run(&z, RSTRING(src)->ptr, RSTRING(src)->len, Z_SYNC_FLUSH);
-    zstream_run(&z, "", 0, Z_FINISH);  /* for checking errors */
-    dst = zstream_detach_buffer(&z);
-    zstream_end(&z);
+    args[0] = (VALUE)&z;
+    args[1] = src;
+    dst = rb_ensure(inflate_run, (VALUE)args, zstream_end, (VALUE)&z);
 
     OBJ_INFECT(dst, src);
     return dst;
@@ -1722,9 +1773,13 @@ static void
 gzfile_free(gz)
     struct gzfile *gz;
 {
-    gz->z.flags |= ZSTREAM_FLAG_FINALIZE;
-    if (ZSTREAM_IS_READY(&gz->z)) {
-	gz->end(gz);
+    struct zstream *z = &gz->z;
+
+    if (ZSTREAM_IS_READY(z)) {
+	if (z->func == &deflate_funcs) {
+	    finalizer_warn("Zlib::GzipWriter object must be closed explicitly.");
+	}
+	zstream_finalize(z);
     }
     free(gz);
 }
@@ -2036,7 +2091,7 @@ gzfile_check_footer(gz)
     if (gz->crc != crc) {
 	rb_raise(cCRCError, "invalid compressed data -- crc error");
     }
-    if (gz->z.stream.total_out != length) {
+    if ((gz->z.stream.total_out & 0xFFFFFFFFUL) != length) {
 	rb_raise(cLengthError, "invalid compressed data -- length error");
     }
 }
@@ -2158,22 +2213,10 @@ gzfile_ungetc(gz, c)
 }
 
 static VALUE
-gzfile_finalize(obj)
-    VALUE obj;
+gzfile_writer_end_run(arg)
+    VALUE arg;
 {
-    struct gzfile *gz = (struct gzfile *)obj;
-    gzfile_write_raw(gz);
-    return Qnil;
-}
-
-static void
-gzfile_writer_end(gz)
-    struct gzfile *gz;
-{
-    int aborted;
-
-    if (ZSTREAM_IS_CLOSED(&gz->z)) return;
-    gz->z.flags |= ZSTREAM_FLAG_CLOSED;
+    struct gzfile *gz = (struct gzfile *)arg;
 
     if (!(gz->z.flags & GZFILE_FLAG_HEADER_FINISHED)) {
 	gzfile_make_header(gz);
@@ -2181,40 +2224,43 @@ gzfile_writer_end(gz)
 
     zstream_run(&gz->z, "", 0, Z_FINISH);
     gzfile_make_footer(gz);
-
-    if (ZSTREAM_IS_FINALIZE(&gz->z)) {
-	if (NIL_P(gz->io)) return;
-	rb_warn("Zlib::GzipWriter object must be closed explicitly.");
-	if (!SPECIAL_CONST_P(gz->io) && OBJ_IS_FREED(gz->io)) {
-	    aborted = 1;
-	}
-	else {
-	    rb_protect(gzfile_finalize, (VALUE)gz, &aborted);
-	}
-	if (aborted) {
-	    rb_warn("gzip footer is not written; broken gzip file");
-	}
-	zstream_end(&gz->z);
-	return;
-    }
     gzfile_write_raw(gz);
-    zstream_end(&gz->z);
+
+    return Qnil;
+}
+
+static void
+gzfile_writer_end(gz)
+    struct gzfile *gz;
+{
+    if (ZSTREAM_IS_CLOSING(&gz->z)) return;
+    gz->z.flags |= ZSTREAM_FLAG_CLOSING;
+
+    rb_ensure(gzfile_writer_end_run, (VALUE)gz, zstream_end, (VALUE)&gz->z);
+}
+
+static VALUE
+gzfile_reader_end_run(arg)
+    VALUE arg;
+{
+    struct gzfile *gz = (struct gzfile *)arg;
+
+    if (GZFILE_IS_FINISHED(gz)
+	&& !(gz->z.flags & GZFILE_FLAG_FOOTER_FINISHED)) {
+	gzfile_check_footer(gz);
+    }
+
+    return Qnil;
 }
 
 static void
 gzfile_reader_end(gz)
     struct gzfile *gz;
 {
-    if (ZSTREAM_IS_CLOSED(&gz->z)) return;
-    gz->z.flags |= ZSTREAM_FLAG_CLOSED;
+    if (ZSTREAM_IS_CLOSING(&gz->z)) return;
+    gz->z.flags |= ZSTREAM_FLAG_CLOSING;
 
-    if (GZFILE_IS_FINISHED(gz)
-	&& !ZSTREAM_IS_FINALIZE(&gz->z)
-	&& !(gz->z.flags & GZFILE_FLAG_FOOTER_FINISHED)) {
-	gzfile_check_footer(gz);
-    }
-
-    zstream_end(&gz->z);
+    rb_ensure(gzfile_reader_end_run, (VALUE)gz, zstream_end, (VALUE)&gz->z);
 }
 
 static void
@@ -2450,7 +2496,7 @@ rb_gzfile_set_mtime(obj, mtime)
 	rb_raise(cGzError, "header is already written");
     }
 
-    if (FIXNUM_P(time)) {
+    if (FIXNUM_P(mtime)) {
 	gz->mtime = FIX2INT(mtime);
     }
     else {
@@ -2976,7 +3022,7 @@ rb_gzreader_readchar(obj)
     VALUE dst;
     dst = rb_gzreader_getc(obj);
     if (NIL_P(dst)) {
-	rb_raise(rb_eEOFError, "End of file reached");
+	rb_raise(rb_eEOFError, "end of file reached");
     }
     return dst;
 }
@@ -3075,6 +3121,8 @@ gzreader_gets(argc, argv, obj)
     if (NIL_P(rs)) {
 	dst = gzfile_read_all(gz);
 	if (RSTRING(dst)->len != 0) gz->lineno++;
+	else
+		return Qnil;
 	return dst;
     }
 
@@ -3158,7 +3206,7 @@ rb_gzreader_readline(argc, argv, obj)
     VALUE dst;
     dst = rb_gzreader_gets(argc, argv, obj);
     if (NIL_P(dst)) {
-	rb_raise(rb_eEOFError, "End of file reached");
+	rb_raise(rb_eEOFError, "end of file reached");
     }
     return dst;
 }
@@ -3333,7 +3381,7 @@ void Init_zlib()
     rb_define_singleton_method(cDeflate, "deflate", rb_deflate_s_deflate, -1);
     rb_define_alloc_func(cDeflate, rb_deflate_s_allocate);
     rb_define_method(cDeflate, "initialize", rb_deflate_initialize, -1);
-    rb_define_method(cDeflate, "clone", rb_deflate_clone, 0);
+    rb_define_method(cDeflate, "initialize_copy", rb_deflate_init_copy, 1);
     rb_define_method(cDeflate, "deflate", rb_deflate_deflate, -1);
     rb_define_method(cDeflate, "<<", rb_deflate_addstr, 1);
     rb_define_method(cDeflate, "flush", rb_deflate_flush, -1);
