@@ -3,7 +3,7 @@
   marshal.c -
 
   $Author: shyouhei $
-  $Date: 2009-01-28 21:12:07 +0900 (Wed, 28 Jan 2009) $
+  $Date: 2010-06-23 22:16:28 +0900 (Wed, 23 Jun 2010) $
   created at: Thu Apr 27 16:30:01 JST 1995
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -1451,21 +1451,21 @@ marshal_load(argc, argv)
     VALUE *argv;
 {
     VALUE port, proc;
-    int major, minor;
+    int major, minor, taint = Qfalse;
     VALUE v;
     struct load_arg arg;
 
     rb_scan_args(argc, argv, "11", &port, &proc);
     v = rb_check_string_type(port);
     if (!NIL_P(v)) {
-	arg.taint = OBJ_TAINTED(port); /* original taintedness */
+	taint = OBJ_TAINTED(port); /* original taintedness */
 	port = v;
     }
     else if (rb_respond_to(port, s_getc) && rb_respond_to(port, s_read)) {
 	if (rb_respond_to(port, s_binmode)) {
 	    rb_funcall2(port, s_binmode, 0, 0);
 	}
-	arg.taint = Qtrue;
+	taint = Qtrue;
     }
     else {
 	rb_raise(rb_eTypeError, "instance of IO needed");
@@ -1476,6 +1476,7 @@ marshal_load(argc, argv)
     arg.data    = st_init_numtable();
     arg.proc = 0;
     arg.wrapper = Data_Wrap_Struct(rb_cData, mark_load_arg, 0, &arg);
+    arg.taint = taint;
 
     major = r_byte(&arg);
     minor = r_byte(&arg);
